@@ -246,25 +246,30 @@ class AutoLogin_Container_LDAP extends AutoLogin_Container
             $encryptpwds = $this->_to_array($this->prop[$this->options['encryptpwdattr']]);
 
             for ($i = 0; $i < count($encryptpwds); $i++) {
-                 if ($app) {
-                     if (preg_match("/^{".$app."}(.+)/i", $encryptpwds[$i], $matches)) {
-                         $password = $this->decrypt($matches[1]);
-                     }
-                 } else {
-                     if (!preg_match("/^{[^}]+}/", $encryptpwds[$i])) {
-                         $password = $this->decrypt($encryptpwds[$i]);
-                     }
-                 }
-             }
+                if ($app) {
+                    if (preg_match("/^{".$app."}(.+)/i", $encryptpwds[$i], $matches)) {
+                        $password = $this->decrypt($matches[1]);
+                        break;
+                    }
+                }
+                if (!preg_match("/^{[^}]+}/", $encryptpwds[$i])) {
+                    $password = $this->decrypt($encryptpwds[$i]);
+                }
+            }
         }
 
         return $password;
     }
     // }}}
 
-    function getTenants()
+    function getTenants($tenant = null)
     {
-        $res = @ldap_list($this->conn, $this->options['basedn'], "(".$this->options['tenantattr']."=*)");
+        if ($tenant) {
+            $filter = "(".$this->options['tenantattr']."=$tenant)";
+        } else {
+            $filter = "(".$this->options['tenantattr']."=*)";
+        }
+        $res = @ldap_list($this->conn, $this->options['basedn'], $filter, array($this->options['tenantattr'], 'seciossallowedfunction'));
         if ($res == false) {
             return PEAR::raiseError(ldap_error($this->conn), ldap_errno($this->conn));
         }
@@ -272,7 +277,7 @@ class AutoLogin_Container_LDAP extends AutoLogin_Container
         $entries = ldap_get_entries($this->conn, $res);
         $tenants = array();
         for ($i = 0; $i < $entries['count']; $i++) {
-            $tenants[] = $entries[$i][$this->options['tenantattr']][0];
+            $tenants[$entries[$i][$this->options['tenantattr']][0]] = array('func' => isset($entries[$i]['seciossallowedfunction']) ? $entries[$i]['seciossallowedfunction'] : array());
         }
 
         return $tenants;
@@ -433,6 +438,37 @@ class AutoLogin_Container_LDAP extends AutoLogin_Container
     }
     // }}}
 
+    function deletePassword($app)
+    {
+        if (!$app) {
+            return true;
+        }
+
+        if (isset($this->prop[$this->options['encryptpwdattr']])) {
+            $encryptpwds = $this->_to_array($this->prop[$this->options['encryptpwdattr']]);
+
+            for ($i = 0; $i < count($encryptpwds); $i++) {
+                if (preg_match("/^{".$app."}/i", $encryptpwds[$i])) {
+                    if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
+                        $this->_connect();
+                    }
+
+                    if (!@ldap_mod_del($this->conn, $this->id, array($this->options['encryptpwdattr'] => $encryptpwds[$i]))) {
+                        return PEAR::raiseError(ldap_error($this->conn), ldap_errno($this->conn));
+                    }
+
+                    if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
+                        $this->_disconnect();
+                    }
+
+                    return true;
+                }
+            }
+        }
+
+        return true;
+    }
+
     // {{{ getPwdChangedTime()
     /**
      *  パスワード変更時間を取得する
@@ -606,18 +642,33 @@ class AutoLogin_Container_LDAP extends AutoLogin_Container
      *  @param  string   シークレット
      *  @return mixed    true:正常終了 PEAR_Error:エラー
      */
-    function setSecret($secret = null)
+    function setSecret($secret, $pin = null)
     {
         if (!$secret || strlen($secret) > 255) {
             return PEAR::raiseError("Invalid secret", AUTO_LOGIN_INVALID_VALUE);
         }
 
+        $prop = array();
+        if (array_search('seciossOtpUser', $this->prop['objectclass']) === false) {
+            $prop['objectclass'] = $this->prop['objectclass'];
+            array_push($prop['objectclass'], 'seciossOtpUser');
+        }
+
         $encrypt = $this->encrypt($secret);
         if (PEAR::isError($encrypt)) {
             return $encrypt;
         }
+        $prop[$this->options['secretattr']] = $encrypt;
 
-        return $this->set($this->options['secretattr'], $encrypt);
+        if ($pin) {
+            $encrypt = $this->encrypt($pin);
+            if (PEAR::isError($encrypt)) {
+                return $encrypt;
+            }
+            $prop['seciossotpsecretpin'] = $encrypt;
+        }
+
+        return $this->setProp($prop);
     }
     // }}}
 
